@@ -262,37 +262,34 @@ get_standalone_layer (MetaWindow *window)
   return layer;
 }
 
+typedef struct
+{
+  MetaStackLayer max;
+} MaxLayerData;
+
+static gboolean
+max_layer_func (MetaWindow *window,
+                void       *data)
+{
+  MaxLayerData *d = data;
+  MetaStackLayer layer;
+  
+  layer = get_standalone_layer (window);
+  if (layer > d->max)
+    d->max = layer;
+
+  return TRUE;
+}
+
 static MetaStackLayer
 get_maximum_layer_of_ancestor (MetaWindow *window)
 {
-  MetaWindow *w;
-  MetaStackLayer max;
-  MetaStackLayer layer;
-  
-  max = get_standalone_layer (window);
-  
-  w = window;
-  while (w != NULL)
-    {
-      if (w->xtransient_for == None ||
-          w->transient_parent_is_root_window)
-        break;
-      
-      w = meta_display_lookup_x_window (w->display, w->xtransient_for);
-      
-      if (w == window)
-        break; /* Cute, someone thought they'd make a transient_for cycle */
-      
-      /* w may be null... */
-      if (w != NULL)
-        {
-          layer = get_standalone_layer (w);
-          if (layer > max)
-            max = layer;
-        }
-    }
+  MaxLayerData d;
 
-  return max;
+  d.max = get_standalone_layer (window);
+  meta_window_foreach_ancestor (window, max_layer_func, &d);
+  
+  return d.max;
 }
 
 /* Note that this function can never use window->layer only
@@ -501,6 +498,8 @@ add_constraint (Constraint **constraints,
                 MetaWindow  *below)
 {
   Constraint *c;
+
+  g_assert (above->screen == below->screen);
   
   /* check if constraint is a duplicate */
   c = constraints[below->stack_position];
@@ -575,7 +574,8 @@ create_constraints (Constraint **constraints,
             {
               MetaWindow *group_window = tmp2->data;
 
-              if (!WINDOW_IN_STACK (group_window))
+              if (!WINDOW_IN_STACK (group_window) ||
+                  w->screen != group_window->screen)
                 {
                   tmp2 = tmp2->next;
                   continue;
@@ -610,7 +610,8 @@ create_constraints (Constraint **constraints,
           parent =
             meta_display_lookup_x_window (w->display, w->xtransient_for);
 
-          if (parent && WINDOW_IN_STACK (parent))
+          if (parent && WINDOW_IN_STACK (parent) &&
+              parent->screen == w->screen)
             {
               meta_topic (META_DEBUG_STACK, "Constraining %s above %s due to transiency\n",
                           w->desc, parent->desc);
